# R script for PERMDISP test 
# test the significance between observed jaccard's similarity and expected jaccard's similarity under null model
# betadisper object is needed
# group is also necessary which contains the reapeat numbers of each group
# //////////////////
# Li, Chaonan (CIBCAS, chaonanl@yahoo.com)
# ////////////////
# ////start////#

permDispDiff <- function(observedMod, expectedMod, groups, n=999){
  require(vegan)
  # first we create a function for difference test within group using PERMDISP 
  permDisp <- function(object, nperm = n){
    nobs <- length(object$distances)
    mod <- lm(object$distances ~ object$group)
    mod.Q <- mod$qr
    p <- mod.Q$rank
    resids <- qr.resid(mod.Q, object$distances)
    res <- numeric(length = nperm + 1)
    res[1] <- summary(mod)$fstatistic[1]
    for(i in seq(along = res[-1])) {
      perm.resid <- sample(resids)
      f <- qr.fitted(mod.Q, perm.resid)
      mss <- sum((f - mean(f))^2)
      r <- qr.resid(mod.Q, perm.resid)
      rss <- sum(r^2)
      rdf <- nobs - p
      resvar <- rss / rdf
      res[i+1] <- (mss / (p - 1)) / resvar
    }
    pval <- sum(res >= res[1]) / length(res)
    mod.aov <- anova(object)
    retval <- cbind(mod.aov[, 1:4], c(nperm, NA), c(pval, NA))
    dimnames(retval) <- list(c("Groups", "Residuals"),
                             c("Df", "Sum Sq", "Mean Sq", "F", "N.Perm",
                               "P")) 
    class(retval) <- c("permDisp", class(retval))
    retval
  }
  #/////
  # here we create a unreal betadisper object to cheat the perdisp function
  grObs <- observedMod$group
  disObs <-observedMod$distances # extract the initial values
  grExp <- expectedMod$group
  disExp <- expectedMod$distances
  groupLevel <- levels(factor(grObs))
  a <- 1
  b <- 0
  perdispOut <- NULL
  for (i in seq(1:length(groupLevel))){
    #print(i)
    new_mod <- observedMod
    gr <- groupLevel[i] #the current group name
    gr1 <- groups[i]
    b <- b + gr1
    groupNameObs <- paste("Obs",gr,sep="_")
    groupNameExp <- paste("Exp",gr,sep="_")
    new_group <- c(rep(groupNameObs,gr1),rep(groupNameExp,gr1))
    valueObs <- disObs[a:b]
    valueExp <- disExp[a:b]
    new_value <- c(valueObs,valueExp)
    new_mod$distances <- new_value
    new_mod$group <- new_group
    perdispRes <- permDisp(new_mod)
    f.value <- perdispRes$F[1]
    p.value <- perdispRes$P[1]
    valueObsOut <- mean(valueObs)
    valueExpOut <- mean(valueExp)
    res.value <- c(gr, valueObsOut,valueExpOut,f.value, p.value)
    perdispOut <- rbind(perdispOut, res.value)
    a <- a + gr1
  }
  colnames(perdispOut) <- c("Groups","Obs_Centroid","Exp_Centroid","F_value","p_value")
  rownames(perdispOut) <- perdispOut[,1]
  perdispOut <- perdispOut[,-1]
  perdispOut
}

#end/////